@@ -9,19 +9,17 @@
 //    This file is part of the omniORB library
 //
 //    The omniORB library is free software; you can redistribute it and/or
-//    modify it under the terms of the GNU Library General Public
+//    modify it under the terms of the GNU Lesser General Public
 //    License as published by the Free Software Foundation; either
-//    version 2 of the License, or (at your option) any later version.
+//    version 2.1 of the License, or (at your option) any later version.
 //
 //    This library is distributed in the hope that it will be useful,
 //    but WITHOUT ANY WARRANTY; without even the implied warranty of
 //    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//    Library General Public License for more details.
+//    Lesser General Public License for more details.
 //
-//    You should have received a copy of the GNU Library General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
-//    02111-1307, USA
+//    You should have received a copy of the GNU Lesser General Public
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 //
 // Description:
@@ -601,8 +599,8 @@ omniOrbBOA::dispatch(omniCallHandle& handle, omniLocalIdentity* id)
 
   if( omniORB::traceInvocations ) {
     omniORB::logger l;
-    l << "Dispatching remote call \'" 
-      << handle.operation_name() << "\' to: "
+    l << "Dispatching remote call '" 
+      << handle.operation_name() << "' to: "
       << id << '\n';
   }
 
@@ -613,6 +611,12 @@ omniOrbBOA::dispatch(omniCallHandle& handle, omniLocalIdentity* id)
 		    CORBA::COMPLETED_NO);
     }
   }
+  if( omniORB::traceInvocationReturns ) {
+    omniORB::logger l;
+    l << "Return from remote call '"
+      << handle.operation_name() << "' to: "
+      << id << '\n';
+  }
 }
 
 
@@ -1157,7 +1161,7 @@ omniORB::generateNewKey(omniORB::objectKey& k)
 #ifndef __BCPLUSPLUS__
       struct _timeb v;
       _ftime(&v);
-      omniORB_seed.hi = v.time;
+      omniORB_seed.hi = (CORBA::ULong)v.time;
       omniORB_seed.med = v.millitm + _getpid();
       omniORB_seed.lo = 0;
 #else
