@@ -9,19 +9,17 @@
 //    This file is part of the omniORB library
 //
 //    The omniORB library is free software; you can redistribute it and/or
-//    modify it under the terms of the GNU Library General Public
+//    modify it under the terms of the GNU Lesser General Public
 //    License as published by the Free Software Foundation; either
-//    version 2 of the License, or (at your option) any later version.
+//    version 2.1 of the License, or (at your option) any later version.
 //
 //    This library is distributed in the hope that it will be useful,
 //    but WITHOUT ANY WARRANTY; without even the implied warranty of
 //    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//    Library General Public License for more details.
+//    Lesser General Public License for more details.
 //
-//    You should have received a copy of the GNU Library General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
-//    02111-1307, USA
+//    You should have received a copy of the GNU Lesser General Public
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 //
 // Description:
@@ -157,6 +155,11 @@ giopImpl12::inputQueueMessage(giopStream* g, giopStream_Buffer* b) {
   CORBA::ULong reqid;
 
   if (mtype != GIOP::CloseConnection && mtype != GIOP::MessageError) {
+    if (b->size < 16) {
+      giopStream_Buffer::deleteBuffer(b);
+      OMNIORB_THROW(MARSHAL, MARSHAL_PassEndOfMessage,
+                    (CORBA::CompletionStatus)g->completion());
+    }
 
     // unmarshal request id.
     reqid = *((CORBA::ULong*)(hdr + 12));
@@ -173,6 +176,7 @@ giopImpl12::inputQueueMessage(giopStream* g, giopStream_Buffer* b) {
     inputTerminalProtocolError(g, __FILE__, __LINE__,
 			       "Received a MessageError message");
     // never reach here
+    reqid = 0; // avoid compiler warning
   }
   else if (g->pd_strand->isClient() || g->pd_strand->isBiDir()) {
     // orderly shutdown.
@@ -189,11 +193,13 @@ giopImpl12::inputQueueMessage(giopStream* g, giopStream_Buffer* b) {
 				    "Orderly connection shutdown",
 				    g->pd_strand);
     // never reach here
+    reqid = 0; // avoid compiler warning
   }
   else {
     giopStream_Buffer::deleteBuffer(b);
     inputTerminalProtocolError(g, __FILE__, __LINE__,
 			       "Orderly connection shutdown on server");
+    reqid = 0; // avoid compiler warning
     // never reach here
   }
 
@@ -518,6 +524,7 @@ giopImpl12::inputNewFragment(giopStream* g) {
 
   g->pd_inb_mkr = (void*)(hdr + 16);
   g->pd_inb_end = (void*)g->inputBufferLast();
+  OMNIORB_ASSERT(g->pd_inb_mkr <= g->pd_inb_end);
 
   g->inputExpectAnotherFragment(((hdr[6] & 0x2) ? 1 : 0));
 
@@ -601,6 +608,7 @@ giopImpl12::inputReplyBegin(giopStream* g,
 
   g->pd_inb_mkr = (void*)(hdr + 16);
   g->pd_inb_end = (void*)g->inputBufferLast();
+  OMNIORB_ASSERT(g->pd_inb_mkr <= g->pd_inb_end);
 
   g->inputExpectAnotherFragment(((hdr[6] & 0x2) ? 1 : 0));
 
@@ -900,7 +908,10 @@ giopImpl12::unmarshalWildCardRequestHeader(giopStream* g) {
 	l << "Server has closed a bi-directional connection on strand "
 	  << (void*)g->pd_strand << ". Will scavenge it.\n";
       }
-      g->pd_strand->startIdleCounter();
+      {
+	omni_tracedmutex_lock sync(*omniTransportLock);
+	g->pd_strand->startIdleCounter();
+      }
     }
     inputRaiseCommFailure(g, "Orderly connection shutdown");
     break;
@@ -1785,8 +1796,8 @@ giopImpl12::sendLocateReply(giopStream* g,GIOP::LocateStatusType rc,
     break;
   }
 
-  int repoid_size;
-  const char* repoid;
+  int         repoid_size = 0;
+  const char* repoid      = 0;
   
   // Compute and initialise the message size field
   {
