@@ -9,19 +9,17 @@
 //    This file is part of the omniORB library
 //
 //    The omniORB library is free software; you can redistribute it and/or
-//    modify it under the terms of the GNU Library General Public
+//    modify it under the terms of the GNU Lesser General Public
 //    License as published by the Free Software Foundation; either
-//    version 2 of the License, or (at your option) any later version.
+//    version 2.1 of the License, or (at your option) any later version.
 //
 //    This library is distributed in the hope that it will be useful,
 //    but WITHOUT ANY WARRANTY; without even the implied warranty of
 //    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//    Library General Public License for more details.
+//    Lesser General Public License for more details.
 //
-//    You should have received a copy of the GNU Library General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-//    02111-1307, USA
+//    You should have received a copy of the GNU Lesser General Public
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 
 #include <omniORB4/CORBA.h>
@@ -68,6 +66,7 @@ CORBA::ULong orbParameters::inConScanPeriod = 180;
 //   Valid values = (n >= 0 in seconds) 
 //                   0 --> do not close idle connections.
 
+
 ////////////////////////////////////////////////////////////////////////
 class Scavenger : public omniTask {
 public:
@@ -530,6 +529,14 @@ giopStrand::releaseServer(IOP_S* iop_s)
       }
     }
   }
+  else if (isBiDir() && pd_state == DYING) {
+    if (omniORB::trace(25)) {
+      omniORB::logger log;
+      log << "Bi-directional strand " << (void*)this
+          << " is dying after handling a callback.\n";
+    }
+    startIdleCounter();
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -608,7 +615,7 @@ Scavenger::removeIdle(StrandList& src,StrandList& dest,
 
     if ( s->idlebeats >= 0 ) {
 
-      if (omniORB::trace(30)) {
+      if (omniORB::trace(29)) {
 	omniORB::logger log;
 	log << "Scavenger reduce idle count for strand "
 	    << (void*)s << " to " << (s->idlebeats - 1) << "\n";
@@ -682,9 +689,13 @@ Scavenger::execute()
     }
 
     // Now go through the list to delete them all
+    CORBA::Boolean scavenged_client_strands = 0;
     {
       StrandList* p = client_shutdown_list.next;
       while ( p != &client_shutdown_list ) {
+
+        scavenged_client_strands = 1;
+
 	giopStrand* s = (giopStrand*)p;
 	p = p->next;
 	s->StrandList::remove();
@@ -694,10 +705,20 @@ Scavenger::execute()
           // message.
           sendCloseConnection(s);
         }
-	s->safeDelete(1);
+	{
+	  omni_optional_lock sync(*omniTransportLock,
+                                  !s->isBiDir(), !s->isBiDir());
+	  s->safeDelete(1);
+	}
       }
     }
 
+    if (scavenged_client_strands) {
+      // We have scavenged at least one client strand, so we reset any
+      // idle giopRope address orders.
+      giopRope::resetIdleRopeAddresses();
+    }
+    
     {
       // We have to hold <omniTransportLock> while disposing of the
       // server strands, since other threads may be dealing with them.
@@ -794,8 +815,9 @@ public:
 			1,
 			"-ORBscanGranularity < n >= 0 sec >") {}
 
-  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
-
+  void visit(const char* value,orbOptions::Source)
+    OMNI_THROW_SPEC (orbOptions::BadParam)
+  {
     CORBA::ULong v;
     if (!orbOptions::getULong(value,v)) {
       throw orbOptions::BadParam(key(),value,
@@ -822,8 +844,9 @@ public:
 			1,
 			"-ORBoutConScanPeriod < n >= 0 sec >") {}
 
-  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
-
+  void visit(const char* value,orbOptions::Source)
+    OMNI_THROW_SPEC (orbOptions::BadParam)
+  {
     CORBA::ULong v;
     if (!orbOptions::getULong(value,v)) {
       throw orbOptions::BadParam(key(),value,
@@ -850,8 +873,9 @@ public:
 			1,
 			"-ORBinConScanPeriod < n >= 0 sec >") {}
 
-  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
-
+  void visit(const char* value,orbOptions::Source)
+    OMNI_THROW_SPEC (orbOptions::BadParam)
+  {
     CORBA::ULong v;
     if (!orbOptions::getULong(value,v)) {
       throw orbOptions::BadParam(key(),value,
